
#define BUF_SIZE (sizeof(MAILBOX))

/// NOTE: MsgE2B[0] and MsgB2E[0] isn't currently used, but is allocated just to keep things nice and even and symmetrical
typedef struct {
	HWND	hWnd[8];		// Windows HWND for Emulator and each Buddy
	DWORD	Model;			// set by Emulator to indicate current emulation: 0=85A, 1=85B, 2=87, 0xFFFFFFFF=not currently active (still in boot-up phase)

	/// For BMSG_* messages sent by Emulator to Buddies
	BYTE	MsgE2B[8];		// 1 Mailbox per Buddy (0 isn't used)
	DWORD	ArgE2B[8][4];	// 16 bytes of message arguments per Buddy
	BYTE	BufE2B[8][256];	// 256 bytes of message data per Buddy

	/// For MSG_* messages sent by Buddies to Emulator
	BYTE	MsgB2E[8];		// 1 Mailbox per Buddy (0 isn't used)
	DWORD	ArgB2E[8][4];	// 16 bytes per of message arguments per Buddy
	BYTE	BufB2E[8][256];	// 256 bytes of message data per Buddy

} MAILBOX;

///========= MESSAGES =================
#define	BMSG_PWON		1	// sent when Emulator starts up and/or is changing Models
#define	BMSG_PWOFF		2	// sent when Emulator shuts down and/or is changing Models
#define	BMSG_RESET		3	// sent when Emulator is doing RESET TO POWER-ON and/or is changing Models
#define	BMSG_GONE		4	// sent when Emulator or Buddy is terminating (in the case of the Emulator, is preceded by BMSG_PWOFF
#define	BMSG_HALT		5	// sent by a Buddy to tell the Emulator to hang (if ArgB2E[0]==1), just answering messages from it, or resume (if ArgB2E[0]==0), releasing the HALT
#define	BMSG_INT		6	// sent by Buddy to generate an emulated hardware interrupt
#define	BMSG_READMEM	7	// sent by Buddy to read up to 256 bytes of "Series 80 memory" starting at the low-address in ArgB2E[0] (up to 3 byte-address), for the number of bytes in ArgB2E[1] (max 256 bytes), returned in BufB2E[] when MsgB2E[] is set to 0. ArgB2E[2] is set to the # of bytes read.
#define	BMSG_WRITEMEM	8	// sent by Buddy to write up to 256 bytes of "Series 80 memory" starting at the low-address in ArgB2E[0] (up to 3 byte-address), for the number of bytes in ArgB2E[1] (max 256 bytes) from BufB2E[]. ArgB2E[2] is set to the # of bytes written.

#define	BMSG_WANT		9	// sent by a Buddy to tell the Emulator what it wants to hear about
#define	BMSG_KEY		10	// sent by Emulator to Buddies that WANTKEYS when a key is pressed, sent by Buddy to Emulator when Buddy wants to stuff a PC keycode into the Emulator's KEYMAP process
#define	BMSG_TRACE		11	// sent by Emulator to Buddies that WANTTRACE before EVERY instruction
#define	BMSG_CRT		12	// sent by Emulator to Buddies that WANTCRT after every write to a CRT register
#define	BMSG_IO			13	// sent by Emulator to Buddies that WANTIO on every store/load of 177400 to 177777
#define	BMSG_TYPE		14	// sent by Buddy to tell Emulator that a buffer (BufB2E[n]) of ASCII codes (including, possibly, CR) is to be added to the TYPE ASCII buffer for ...uh... typing

///========= WANT FLAGS ===============
/// NOTE: Emulator uses the BWANT_* defines to index into an internal Buddy array.
/// But, when a Buddy is sending the BMSG_WANT message, rather than passing the
/// BWANT_* defines, it must pass an OR'ing of (1<<BWANT_xxx), so that ALL the
/// items the Buddy is interested in hearing about get passed in a single message
#define	BWANT_KEY	0		// BuddyWants[] index for Buddies that want KEYS
#define	BWANT_TRACE	1		// BuddyWants[] index for Buddies that want to know every instruction address executed
#define	BWANT_CRT	2		// BuddyWants[] index for Buddies that want CRT accesses
#define	BWANT_IO	3		// BuddyWants[] index for Buddies that want I/O accesses (and the chance to replace the Emulator's normal response)

#define	BWANTCNT	4		// must be 1 more than the last BWANT_ value

/// The following #defines are pretty much ONLY for the Emulator
#define	IsBuddyMsg()		(*(uint64_t*)(pMailBox->MsgB2E))
#define	AnyBuddyWants(w)	(*(uint64_t*)BuddyWants[w])

/// NOTE: our pointer to the shared MAILBOX is declared as 'volatile' to avoid the C compiler's optimizer from optimizing out code we depend on (such as checking mailboxes in a tight loop)
extern volatile MAILBOX	*pMailBox;
extern int		MailBoxNum;		// this Buddy's mailbox number (1-7)

void StartShare();
void EndShare(BOOL sendGONE);
